.sp-eab-integrations-page-items-wrapper {
    gap: 30px;

    .sp-eap-integrations-card {
        gap: 16px;
        padding: 24px 24px 16px 24px;
        background-color: #ffffff;
        box-shadow: 0px 1px 2px 0px #11111133;
        border-radius: 12px;
    }

    .sp-eap-integrations-card__label {
        margin: 0;
        font-weight: 600;
        font-size: 16px;
        line-height: 1.2;
    }

    .sp-eap-integrations-card__desc {
        color: #757575;
        font-weight: 400;
        font-size: 14px;
        line-height: 1.6;
    }

    .sp-eap-integrations-card__links {
        margin-top: 12px;

        a {
            padding: 4px 10px;
            background-color: #FBFCFF;
            border: 1px solid #ECEDF0;
            border-radius: 4px;
            transition: all 0.4s ease-in-out;

            svg path {
                transition: all 0.4s ease-in-out;
            }

            &:hover {
                color: #fff;
                background-color: #757575;
                border-color: #757575;

                .eap-video-link-icon path {
                    fill: #fff;
                }

                .eap-docs-link-icon path,
                .eap-demo-link-icon path {
                    stroke: #fff;
                }
            }
        }

        a,
        a:focus {
            color: #757575;
        }
    }
}

.sp-eap-modules-page {
    .eaf-pro-modules {
        margin-top: 48px;
    }

    .eaf-pro-modules-indicator-button {
        padding: 17px 24px;
        color: #fff;
        background-color: #02B472;
        border: none;
        border-radius: 4px;
        font-weight: 600;
        font-size: 16px;
        line-height: 1.2;
        transition: all 0.4s ease-in-out;
        cursor: pointer;
    }
}

.eap-dashboard-page-header {
    margin-bottom: 32px;
    margin-top: -4px;

    .eap-dashboard-page-header__label {
        font-weight: 500;
        font-size: 24px;
        line-height: 1;
    }

    .eap-dashboard-page-header__desc {
        font-weight: 400;
        font-size: 14px;
        line-height: 1.3;
    }

    &.eap-upcoming-modules {
        margin-top: 30px;
    }
}

// modal content.
.sp-eap-integrations__modal {
    .components-modal__content {
        width: 960px;
        padding: 32px 48px;
    }

    .components-modal__header {
        padding: 30px 48px;
        border-bottom: 1px solid #DDDDDD;

        .components-modal__header-heading {
            font-weight: 600;
            font-size: 24px;
            line-height: 1.2;
        }

        .components-button:focus {
            box-shadow: none;
        }
    }

    // modal input content wrapper.
    .sp-eap-integrations__modal-content-wrapper {
        gap: 24px;
        background: #FAFAFA;
        padding: 25px;
        border: 1px solid #F0F0F0;
        border-radius: 6px;
    }

    .sp-eap-integrations__modal-input-label {
        font-weight: 600;
        font-size: 14px;
        line-height: 1.2;
        margin: 0 0 8px 0;
    }

    .sp-eap-integrations__modal-input {
        height: 40px;
        width: 100%;
        padding: 12px;
        border: 1px solid #dcdcde;
        border-radius: 4px;
        font-size: 14px;
        line-height: 1;

        &:focus {
            border-color: #dcdcde;
            box-shadow: none;
            outline: none;
        }

        &.max-token {
            width: 150px;
        }
    }

    .sp-eap-integration-input-help-text {
        color: #757575;
        font-weight: 400;
        font-size: 13px;
        line-height: 1.2;

        a {
            text-decoration: none;
        }
    }

    // captcha.
    .sp-eap-recaptcha-dashboard-links {
        font-size: 14px;
        line-height: 1.2;
        margin-bottom: 24px;
    }

    .sp-eap-settings-save-wrapper {
        justify-content: start;
    }

    .sp-eap-integrations__modal-field-row {
        gap: 24px;
    }

    .sp-eap-site-secret-key-wrapper {
        width: 50%;
    }

    .sp-eap-site-secret-key-label {
        display: block;
        margin-bottom: 6px;
    }

    // chat gpt api key.
    .sp-eap-integrations__modal-chatgpt-api-key-page {
        .sp-eap-integrations__modal-field-wrapper {
            width: 70%;

            select {
                max-width: 100%;
            }
        }
    }
}

.sp-eap-dashboard-upcoming-text {
    color: #ffff;
    background-color: #641DD7;
    line-height: 1;
    font-weight: 500;
    margin-left: 10px;

    &.large {
        padding: 5px 10px;
        border-radius: 99px;
        font-size: 14px;
    }

    &.small {
        padding: 4px 6px;
        border-radius: 99px;
        font-size: 11px;
    }
}

.sp-eap-upcoming-card {
    cursor: default;
    pointer-events: none;
}