$opacity-transition : opacity 0.25s ease;

/*
 * BLOCK VISIBILITY TOGGLE PAGE CSS.
 */
.sp-eab-visibility-page {

    // VISIBILITY PAGE TITLE.
    .sp-eab-visibility-page-title {
        margin: 0 0 24px 0;
        font-size: 22px;
        font-weight: 600;
        line-height: 28px;
    }

    // BLOCK LIST WRAPPER.
    .sp-eab-all-block-list {
        gap: 30px;
    }

    // VISIBILITY CARD CSS.
    .sp-eab-visibility-setting-card {
        background-color: #ffffff;
        padding: 17px;
        border-radius: 6px;
        box-shadow: 0 1px 2px 0 hsla(0,0%,7%,.2);
        position: relative;
    }

    // VISIBILITY CARD ICON CSS.
    .sp-eab-visibility-setting-card-icon {
        width: 48px;
        padding: 6px;
        border: 2px solid #FFE0B3;
        border-radius: 4px;
    }

    // VISIBILITY CARD DOCS CSS.
    .sp-eab-visibility-setting-card-docs {
        margin-left: 10px;

        li,
        h4,
        ul {
            margin: 0;
            padding: 0;
        }

        h4 {
            font-size: 16px;
            color: #1E1E1E;
            font-weight: 600;
            line-height: 20px;
            margin-bottom: 4px;
        }

        ul li a {
            text-decoration: none;
            color: #757575;
            font-size: 13px;
            font-weight: 400;
            gap: 5px;
            line-height: 20px;

            &:hover {
                color: var(--eab-primary-color);

                [stroke="#757575"] {
                    stroke: var(--eab-primary-color);
                }
            }
        }

        ul li a:focus {
            box-shadow: none;
        }
    }

    // UPCOMING BLOCK CSS.
    .sp-eab-upcoming-block-title {
        margin-top: 40px;
    }

    .sp-eab-upcoming-block {

        .sp-eab-doc-link,
        .sp-eab-demo-link,
        .sp-eab-visibility-setting-toggle {
            pointer-events: none;
        }
    }
}

.sp-eab-admin-dashboard-wrapper {
    .react-toggle {
        touch-action: pan-x;

        display: inline-block;
        position: relative;
        cursor: pointer;
        background-color: transparent;
        border: 0;
        padding: 0;

        -webkit-touch-callout: none;
        -webkit-user-select: none;
        -khtml-user-select: none;
        -moz-user-select: none;
        -ms-user-select: none;
        user-select: none;

        -webkit-tap-highlight-color: rgba(0, 0, 0, 0);
        -webkit-tap-highlight-color: transparent;
    }

    .react-toggle-screenreader-only {
        border: 0;
        clip: rect(0 0 0 0);
        height: 1px;
        margin: -1px;
        overflow: hidden;
        padding: 0;
        position: absolute;
        width: 1px;
    }

    .react-toggle--disabled {
        cursor: not-allowed;
        opacity: 0.5;
        -webkit-transition: opacity 0.25s;
        transition: opacity 0.25s;
    }

    .react-toggle-track {
        width: 36px;
        height: 18px;
        padding: 0;
        border-radius: 30px;
        background-color: #BBBBBB;
        -webkit-transition: all 0.2s ease;
        -moz-transition: all 0.2s ease;
        transition: all 0.2s ease;
    }

    .react-toggle:hover:not(.react-toggle--disabled) .react-toggle-track {
        background-color: #BBBBBB;
    }

    .react-toggle--checked .react-toggle-track {
        background-color: #4AB866;
    }

    .react-toggle--checked:hover:not(.react-toggle--disabled) .react-toggle-track {
        background-color: #4AB866;
    }

    .react-toggle-track-check {
        position: absolute;
        width: 14px;
        height: 10px;
        top: 0px;
        bottom: 0px;
        margin-top: auto;
        margin-bottom: auto;
        line-height: 0;
        left: 8px;
        opacity: 0;
        -webkit-transition: $opacity-transition;
        -moz-transition: $opacity-transition;
        transition: $opacity-transition;
    }

    .react-toggle--checked .react-toggle-track-check {
        opacity: 1;
        -webkit-transition: $opacity-transition;
        -moz-transition: $opacity-transition;
        transition: $opacity-transition;
    }

    .react-toggle-track-x {
        position: absolute;
        width: 10px;
        height: 10px;
        top: 0px;
        bottom: 0px;
        margin-top: auto;
        margin-bottom: auto;
        line-height: 0;
        right: 10px;
        opacity: 1;
        -webkit-transition: $opacity-transition;
        -moz-transition: $opacity-transition;
        transition: $opacity-transition;
    }

    .react-toggle--checked .react-toggle-track-x {
        opacity: 0;
    }

    .react-toggle-thumb {
        transition: all 0.5s cubic-bezier(0.23, 1, 0.32, 1) 0ms;
        position: absolute;
        top: 2px;
        left: 2px;
        width: 14px;
        height: 14px;
        border: 1px solid #BBBBBB;
        border-radius: 50%;
        background-color: #FAFAFA;

        -webkit-box-sizing: border-box;
        -moz-box-sizing: border-box;
        box-sizing: border-box;

        -webkit-transition: all 0.25s ease;
        -moz-transition: all 0.25s ease;
        transition: all 0.25s ease;
    }

    .react-toggle--checked .react-toggle-thumb {
        left: 20px;
        border-color: #4ab866;
    }

    .react-toggle:active:not(.react-toggle--disabled) .react-toggle-thumb {
        -webkit-box-shadow: 0px 0px 5px 5px #0099E0;
        -moz-box-shadow: 0px 0px 5px 5px #0099E0;
        box-shadow: 0px 0px 5px 5px #0099E0;
    }
}