// Wrapper for saved templates page
.speap-saved-templates-page-wrapper {
    display: flex;
    flex-direction: column;
    gap: 24px;
    width: 100%;
}

// Promo Section
.speap-saved-template-promo {
    background: #ffffff;
    border: 1px solid #ddd;
    border-radius: 6px;
    display: flex;
    align-items: center;
    gap: 48px;
    padding: 30px 48px;

    &__text {
        flex: 1 0 0;
        min-width: 0;
        display: flex;
        flex-direction: column;
        justify-content: center;
        align-items: flex-start;
        gap: 36px;
        padding: 24px 0;
        overflow: hidden;
    }

    &__title-wrap {
        display: flex;
        flex-direction: column;
        align-items: flex-start;
        gap: 16px;
        width: 75%;
        word-break: break-word;
    }

    &__title {
        color: #2c2d2f;
        font-size: 24px;
        font-weight: 600;
        line-height: 1.3;
        margin: 0;
        width: 100%;
        white-space: pre-wrap;
    }

    &__title-accent {
        background: linear-gradient(90deg, #ff512f 0%, #f09819 100%);
        -webkit-background-clip: text;
        background-clip: text;
        -webkit-text-fill-color: transparent;
        color: transparent;
    }

    &__desc {
        color: #4e4f52;
        font-size: 14px;
        font-weight: 400;
        line-height: 20px;
        margin: 0;
        width: 100%;

        strong {
            font-weight: 700;
        }
    }

    &__cta {
        background: #f26c0d;
        border-radius: 4px;
        color: #ffffff;
        cursor: pointer;
        display: inline-flex;
        align-items: center;
        justify-content: center;
                font-size: 16px;
        font-weight: 500;
        line-height: 28px;
        padding: 10px 24px;
        text-decoration: none;
        transition: background 0.2s ease-in-out;
        white-space: nowrap;
        gap: 5px;

        &:hover,
        &:focus {
            background: #da5b00;
            color: #ffffff;
        }
    }

    &__video {
        flex-shrink: 0;
        position: relative;
        width: 400px;
        max-width: 100%;
        aspect-ratio: 1920 / 1080;
        border-radius: 4px;
        overflow: hidden;
    }

    &__video-thumb {
        all: unset;
        position: relative;
        display: block;
        width: 100%;
        height: 100%;
        cursor: pointer;
        background-color: #f26c0d;
        background-position: center;
        background-size: cover;
        background-repeat: no-repeat;
    }

    &__video-overlay {
        position: absolute;
        inset: 0;
        background: rgba(0, 0, 0, 0.3);
    }

    &__video-play {
        position: absolute;
        top: 50%;
        left: 50%;
        transform: translate(-50%, -50%);
        width: 60px;
        height: 60px;
        border-radius: 50%;
        background: #ffffff;
        display: flex;
        align-items: center;
        justify-content: center;
        box-shadow: 0 4px 12px rgba(0, 0, 0, 0.15);

        svg {
            margin-left: 3px;
        }
    }

    &__video-frame {
        width: 100%;
        height: 100%;
        border: 0;
    }
}

@media (max-width: 960px) {
    .speap-saved-template-promo {
        flex-direction: column;
        gap: 32px;
        padding: 32px;

        &__video {
            width: 100%;
        }
    }
}

.sp-eab-saved-templates-page-container {
    background: #ffffff;
    border: 1px solid #ddd;
    border-radius: 6px;
    padding: 32px;
    width: 100%;
    gap: 24px;
}

.sp-eab-saved-template-header {

    .sp-eab-saved-template-header-left {
        gap: 12px;
    }

    .sp-eab-saved-template-search-field {
        background: #fff;
        border: 1px solid #ddd;
        border-radius: 4px;
        height: 36px;
        padding: 6px 6px 6px 16px;
        width: 260px;
    }

    .sp-eab-saved-template-select {
        border: 1px solid #e0e0e0;
        border-radius: 3px;
        height: 36px;
        min-width: 136px;

        &:focus {
            outline: none;
            box-shadow: none;
        }
    }

    .sp-eab-saved-template-select-apply {
        background: #fff;
        border: 1px solid var(--eab-primary-color);
        border-radius: 3px;
        color: var(--eab-primary-color);
        cursor: pointer;
        font-size: 14px;
        font-weight: 500;
        line-height: 20px;
        padding: 6px 16px;
        transition: background .3s ease-in-out, color .3s ease-in-out;
    }
}

.sp-eab-saved-template-add-new {
    padding: 10px 24px 10px 20px;
    border-radius: 4px;
    background: var(--eab-primary-color);
    color: #ffffff;
    font-size: 16px;
    font-weight: 500;
    line-height: 28px;
    text-decoration: none;
    transition: background-color 0.4s ease-in-out;

    &:hover {
        background: #da5b00;
        color: #ffffff;
    }
    &:focus {
        color: #ffffff;
    }
}

.sp-eab-saved-template-content-table {
    display: flex;
    flex-direction: column;
    align-items: flex-start;
    align-self: stretch;
    border-radius: 4px;
    border: 1px solid #FCE2CF;
    background: #FFF;
    box-shadow: 0 3px 6px 0 rgba(0, 0, 0, 0.08);
    overflow: auto;

    button {
        background: transparent;
        border: none;
        cursor: pointer;
        padding: 0;
    }
}

.sp-eab-saved-template-table-head,
.sp-eab-saved-template-table-body {
    display: flex;
    flex-direction: column;
    justify-content: space-between;
    align-items: flex-start;
    align-self: stretch;

    tr {
        display: flex;
        padding-left: 24px;
        align-items: center;
        align-self: stretch;
        justify-content: center;
    }

    .sp-eab-saved-template-table-checkBox {
        input {
            margin-right: 16px;
        }
    }
}

.sp-eab-saved-template-table-title {
    flex: auto;
    a {
        span {
            color: #135e96;
            font-weight: 500;
        }
    }

    a:hover {
        span {
            color: #0165b7;
        }
    }
}

.sp-eab-saved-template-table-editor_type {
    width: 150px;
    .sp-eab-editor-type-badge {
        display: inline-block;
        font-size: 14px;
        font-weight: 500;
        line-height: 1;
        text-transform: capitalize;
    }
}

.sp-eab-saved-template-table-shortcode {
    width: 350px;
    display: flex;
    align-items: center;

    .sp-eab-shortcode-copy-tooltip {
        display: flex;
        align-items: center;
        gap: 3px;
        padding: 5px 8px;
        color: #317d46;
        background-color: rgba(61, 158, 86, 0.1);
        font-size: 14px;
        line-height: 20px;
        font-weight: 500;
        border-radius: 4px;
        margin-left: 5px;
        transition: opacity 0.3s ease;

        svg {
            width: 15px;
            height: 15px;
        }
    }

    > svg {
        width: 16px;
        height: 16px;
        cursor: pointer;
        opacity: 0;
        transform: translateX(-85px);
        transition: opacity 0.3s ease;
        transition: all 0.3s ease;
    }

    &:hover > svg {
        opacity: 1;
    }
}

.sp-eab-saved-template-shortcode-text {
    border-radius: 3px;
    border: 1px solid #EBDBD0;
    background: #F8F4F1;
    padding: 6px 16px;
    display: inline-block;
    cursor: pointer;
    transition: all 0.3s ease;
}

.sp-eab-saved-template-table-date {
    width: 240px;
	text-transform: capitalize;
}

.sp-eab-saved-template-table-action {
    width: 110px;

    >div {
        display: flex;
        gap: 16px;
    }
}

.sp-eab-saved-template-table-head {
    tr {
        background-color: #FAE9E8;
        height: 48px;

        th {
            color: #2f2f2f;
            font-size: 16px;
            font-weight: 500;
            line-height: 20px;
            text-align: left;
            text-transform: capitalize;
        }
    }
}

.sp-eab-saved-template-table-body {
    tr {
        height: 64px;

        &:nth-of-type(even) {
            background: #FDF7F7;
        }
    }

    td * {
        color: #2F2F2F;
        font-size: 14px;
        font-weight: 400;
        line-height: 20px;
    }
}

.sp-eab-saved-template-footer {
    align-items: center;
    display: flex;
    flex-direction: row;
    justify-content: space-between;
}

.sp-eab-saved-template-pagination {
    align-items: center;
    display: flex;
    flex-direction: row;
    gap: 8px;
    justify-content: space-between;
}

.sp-eab-saved-template-pagination-btn {
    align-items: center;
    background-color: transparent;
    border: 1px solid #757575;
    border-radius: 4px;
    box-sizing: border-box;
    color: #2f2f2f;
    display: flex;
    height: 36px;
    justify-content: center;
    width: 36px;
    cursor: pointer;

    &.btn-active {
        background-color: #2f2f2f;
        color: #fff;
    }

    &.btn-disabled {
        border-color: #ddd;

        path {
            fill: #ddd;
        }
    }
}

// #menu-posts-sp_easy_accordion {
//     border-bottom: solid hsla(0, 0%, 100%, .1)
// }